unit uMain;

interface

{$I ver.inc}

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ACRMain, DB, StdCtrls, ComCtrls, DBCtrls, Spin, ExtCtrls, Grids,
  DBGrids;

type
  TfmMain = class(TForm)
    DBGrid1: TDBGrid;
    Panel1: TPanel;
    lbProgress: TLabel;
    Label3: TLabel;
    bnInsert: TButton;
    bnClose: TButton;
    seRecQty: TSpinEdit;
    bnOpenTable: TButton;
    cbLocalDatabase: TCheckBox;
    bnRecNo: TButton;
    DBNavigator1: TDBNavigator;
    reLog: TRichEdit;
    DataSource1: TDataSource;
    ACRTable1: TACRTable;
    ACRDatabase1: TACRDatabase;
    bnEdit: TButton;
    bnDelete: TButton;
    bnRead: TButton;
    gbRemoteParams: TGroupBox;
    gbLocalParams: TGroupBox;
    Label1: TLabel;
    eHost: TEdit;
    Label2: TLabel;
    sePort: TSpinEdit;
    Label4: TLabel;
    eName: TEdit;
    Label5: TLabel;
    eFile: TEdit;
    bnBrowse: TButton;
    OpenDialog1: TOpenDialog;
    procedure FormCreate(Sender: TObject);
    procedure cbLocalDatabaseClick(Sender: TObject);
    procedure bnOpenTableClick(Sender: TObject);
    procedure ACRTable1AfterClose(DataSet: TDataSet);
    procedure ACRTable1AfterOpen(DataSet: TDataSet);
    procedure bnBrowseClick(Sender: TObject);
    procedure bnInsertClick(Sender: TObject);
    procedure bnCloseClick(Sender: TObject);
    procedure bnReadClick(Sender: TObject);
    procedure bnRecNoClick(Sender: TObject);
    procedure bnEditClick(Sender: TObject);
    procedure bnDeleteClick(Sender: TObject);
  private
    { Private declarations }
    FClose: Boolean;
  public
    { Public declarations }
    procedure SetVCLEnabled(Value: Boolean);
    procedure SaveLog;
  end;

var
  fmMain: TfmMain;

implementation

{$R *.dfm}

procedure TfmMain.FormCreate(Sender: TObject);
begin
 Caption := 'Client-Server Basic Speed Test. Accuracer version: '+ACRGetCurrentVersion+'. (c) AidAim Software, http://www.aidaim.com';
 FClose := False;
 ACRDatabase1.Close;
end;

procedure TfmMain.cbLocalDatabaseClick(Sender: TObject);
begin
 if (TCheckBox(Sender).Checked) then
  begin
   gbLocalParams.Enabled := True;
   gbRemoteParams.Enabled := False;
  end
 else
  begin
   gbLocalParams.Enabled := False;
   gbRemoteParams.Enabled := True;
  end;
end;

procedure TfmMain.bnOpenTableClick(Sender: TObject);
begin
 if (ACRTable1.Active) then
  begin
    ACRTable1.Close;
    ACRDatabase1.Close;
    TButton(Sender).Caption := 'Open table';
  end
 else
  begin
    ACRDatabase1.Close;
    ACRDatabase1.LocalDatabase := cbLocalDatabase.Checked;
    ACRDatabase1.DatabaseFileName := eFile.Text;
    ACRDatabase1.ConnectionParams.RemoteHost := eHost.Text;
    ACRDatabase1.ConnectionParams.RemotePort := sePort.Value;
    ACRDatabase1.ConnectionParams.DatabaseName := eName.Text;
    ACRDatabase1.Connected := true;
    if (not ACRTable1.Exists) then
     begin
      ACRTable1.ClearDefinitions;
      ACRTable1.FieldDefs.Add('ID',ftAutoInc);
      ACRTable1.FieldDefs.Add('Str',ftFixedChar,20);
      ACRTable1.IndexDefs.Add('PK','ID',[ixPrimary]);
      ACRTable1.IndexDefs.Add('Idx1','Str',[]);
      ACRTable1.CreateTable;
     end;
    ACRTable1.Open;
    DataSource1.DataSet := ACRTable1;
    TButton(Sender).Caption := 'Close table';
  end;
end;

procedure TfmMain.ACRTable1AfterClose(DataSet: TDataSet);
begin
  bnOpenTable.Caption := 'Open table';
  reLog.Lines.Add('Table closed, LocalDatabase = '+BoolToStr(ACRDatabase1.LocalDatabase,True));
end;

procedure TfmMain.ACRTable1AfterOpen(DataSet: TDataSet);
begin
  bnOpenTable.Caption := 'Close table';
  reLog.Lines.Add('Table opened, RecordCount = '+IntToStr(ACRTable1.RecordCount)+', LocalDatabase = '+BoolToStr(ACRDatabase1.LocalDatabase,True));
end;

procedure TfmMain.bnBrowseClick(Sender: TObject);
begin
  if (OpenDialog1.Execute) then
    eFile.Text := OpenDialog1.FileName;
end;

procedure TfmMain.SetVCLEnabled(Value: Boolean);
begin
  if (Value) then
   begin
    ACRTable1.EnableControls;
   end
  else
   begin
    ACRTable1.DisableControls;
   end;
  bnInsert.Enabled := Value;
  bnEdit.Enabled := Value;
  bnDelete.Enabled := Value;
  bnRead.Enabled := Value;
  bnRecNo.Enabled := Value;
  bnBrowse.Enabled := Value;
  seRecQty.Enabled := Value;
  eFile.Enabled := Value;
  eHost.Enabled := Value;
  eName.Enabled := Value;
  sePort.Enabled := Value;
  reLog.Enabled := Value;
  cbLocalDatabase.Enabled := Value;
end;

procedure TfmMain.bnInsertClick(Sender: TObject);
var i,n,k,c,x:  Integer;
    t:          Cardinal;
    f:          Double;
begin
  if (not ACRTable1.Active) then
   bnOpenTableClick(bnOpenTable);
  SetVCLEnabled(False);
  t := GetTickCount;
  n := seRecQty.Value;
  try
    x := n div 100;
    c := ACRTable1.RecordCount;
    for i := 1 to n do
     begin
      if FClose then Exit;
      k := c+i;
      ACRTable1.Append;
      ACRTable1.FieldByName('Str').AsString := 'test_'+FormatFloat('000000000',k);
      ACRTable1.Post;
      if ((i mod x) = 0) then
       begin
        f := i / ((GetTickCount-t) + 0.000000001);
        lbProgress.Caption := IntToStr(i)+ ' '+FormatFloat('##0.000',f);
        Application.ProcessMessages;
       end;
     end;
  finally
    t := GetTickCount-t;
    f := t / 1000.0;
    reLog.Lines.Add('Insert '+IntToStr(n)+ ' records time, sec: '+#9+FormatFloat('#,##0.000',f));
    SetVCLEnabled(True);
    SaveLog;
  end;
end;

procedure TfmMain.bnCloseClick(Sender: TObject);
begin
  FClose := true;
  Close;
end;

procedure TfmMain.SaveLog;
begin
  reLog.Lines.SaveToFile(IncludeTrailingBackslash(ExtractFilePath(ParamStr(0)))+'log.txt');
end;

procedure TfmMain.bnReadClick(Sender: TObject);
var i,n,x:      Integer;
    t:          Cardinal;
    f:          Double;
    s:          String;
begin
  if (not ACRTable1.Active) then
   bnOpenTableClick(bnOpenTable);
  SetVCLEnabled(False);
  t := GetTickCount;
  n := seRecQty.Value;
  try
    x := n div 100;
    ACRTable1.First;
    i := 0;
    while (not ACRTable1.Eof) and (i < n) do
     begin
      if FClose then Exit;
      s := ACRTable1.FieldByName('Str').AsString;
      Inc(i);
      ACRTable1.Next;
      if ((i mod x) = 0) then
       begin
        f := i / ((GetTickCount-t) + 0.000000001);
        lbProgress.Caption := IntToStr(i)+ ' '+FormatFloat('##0.000',f);
        Application.ProcessMessages;
       end;
     end;
  finally
    t := GetTickCount-t;
    f := t / 1000.0;
    reLog.Lines.Add('Read '+IntToStr(n)+ ' records time, sec: '+#9+FormatFloat('#,##0.000',f));
    SetVCLEnabled(True);
    SaveLog;
  end;
end;

procedure TfmMain.bnRecNoClick(Sender: TObject);
var i,n,x:      Integer;
    t:          Cardinal;
    f:          Double;
begin
  if (not ACRTable1.Active) then
   bnOpenTableClick(bnOpenTable);
  SetVCLEnabled(False);
  t := GetTickCount;
  n := seRecQty.Value;
  try
    x := n div 100;
    for i := 1 to n do
     begin
      if FClose then Exit;
      ACRTable1.RecNo := i;
      if ((i mod x) = 0) then
       begin
        f := i / ((GetTickCount-t) + 0.000000001);
        lbProgress.Caption := IntToStr(i)+ ' '+FormatFloat('##0.000',f);
        Application.ProcessMessages;
       end;
     end;
  finally
    t := GetTickCount-t;
    f := t / 1000.0;
    reLog.Lines.Add('RecNo '+IntToStr(n)+ ' records time, sec: '+#9+FormatFloat('#,##0.000',f));
    SetVCLEnabled(True);
    SaveLog;
  end;
end;

procedure TfmMain.bnEditClick(Sender: TObject);
var i,n,x:      Integer;
    t:          Cardinal;
    f:          Double;
    s:          String;
begin
  if (not ACRTable1.Active) then
   bnOpenTableClick(bnOpenTable);
  SetVCLEnabled(False);
  t := GetTickCount;
  n := seRecQty.Value;
  try
    x := n div 100;
    ACRTable1.First;
    i := 0;
    while (not ACRTable1.Eof) and (i < n) do
     begin
      if FClose then Exit;
      s := ACRTable1.FieldByName('Str').AsString;
      ACRTable1.Edit;
      ACRTable1.FieldByName('Str').AsString := s + ' - Edited!';
      ACRTable1.Post;
      Inc(i);
      ACRTable1.Next;
      if ((i mod x) = 0) then
       begin
        f := i / ((GetTickCount-t) + 0.000000001);
        lbProgress.Caption := IntToStr(i)+ ' '+FormatFloat('##0.000',f);
        Application.ProcessMessages;
       end;
     end;
  finally
    t := GetTickCount-t;
    f := t / 1000.0;
    reLog.Lines.Add('Edit '+IntToStr(n)+ ' records time, sec: '+#9+FormatFloat('#,##0.000',f));
    SetVCLEnabled(True);
    SaveLog;
  end;
end;

procedure TfmMain.bnDeleteClick(Sender: TObject);
var i,n,x:      Integer;
    t:          Cardinal;
    f:          Double;
begin
  if (not ACRTable1.Active) then
   bnOpenTableClick(bnOpenTable);
  SetVCLEnabled(False);
  t := GetTickCount;
  n := seRecQty.Value;
  try
    x := n div 100;
    ACRTable1.First;
    i := 0;
    while (not ACRTable1.Eof) and (i < n) do
     begin
      if FClose then Exit;
      ACRTable1.Delete;
      Inc(i);
      if ((i mod x) = 0) then
       begin
        f := i / ((GetTickCount-t) + 0.000000001);
        lbProgress.Caption := IntToStr(i)+ ' '+FormatFloat('##0.000',f);
        Application.ProcessMessages;
       end;
     end;
  finally
    t := GetTickCount-t;
    f := t / 1000.0;
    reLog.Lines.Add('Delete '+IntToStr(n)+ ' records time, sec: '+#9+FormatFloat('#,##0.000',f));
    SetVCLEnabled(True);
    SaveLog;
  end;
end;

end.
