unit uMain;

interface

{$I ver.inc}

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, DB, StdCtrls, ComCtrls, DBCtrls, Spin, ExtCtrls, Grids,
  DBGrids, edbcomps;

type
  TfmMain = class(TForm)
    DBGrid1: TDBGrid;
    Panel1: TPanel;
    lbProgress: TLabel;
    Label3: TLabel;
    bnInsert: TButton;
    bnClose: TButton;
    seRecQty: TSpinEdit;
    bnOpenTable: TButton;
    bnRecNo: TButton;
    DBNavigator1: TDBNavigator;
    reLog: TRichEdit;
    DataSource1: TDataSource;
    bnEdit: TButton;
    bnDelete: TButton;
    bnRead: TButton;
    gbRemoteParams: TGroupBox;
    Label1: TLabel;
    eHost: TEdit;
    Label2: TLabel;
    sePort: TSpinEdit;
    EDBSession1: TEDBSession;
    EDBDatabase1: TEDBDatabase;
    EDBTable1: TEDBTable;
    EDBQuery1: TEDBQuery;
    procedure FormCreate(Sender: TObject);
    procedure cbLocalDatabaseClick(Sender: TObject);
    procedure bnOpenTableClick(Sender: TObject);
    procedure EDBTable1AfterClose(DataSet: TDataSet);
    procedure EDBTable1AfterOpen(DataSet: TDataSet);
    procedure bnInsertClick(Sender: TObject);
    procedure bnCloseClick(Sender: TObject);
    procedure bnReadClick(Sender: TObject);
    procedure bnRecNoClick(Sender: TObject);
    procedure bnEditClick(Sender: TObject);
    procedure bnDeleteClick(Sender: TObject);
  private
    { Private declarations }
    FClose: Boolean;
  public
    { Public declarations }
    procedure SetVCLEnabled(Value: Boolean);
    procedure SaveLog;
    function IsDBExists: Boolean;
    procedure CreateDB;
  end;

var
  fmMain: TfmMain;

implementation

{$R *.dfm}

procedure TfmMain.FormCreate(Sender: TObject);
begin
 Caption := 'Client-Server Basic Speed Test. ElevateDB version: '+EDBTable1.EngineVersion+'. (c) AidAim Software, http://www.aidaim.com';
 FClose := False;
 EDBTable1.Close;
end;


procedure TfmMain.cbLocalDatabaseClick(Sender: TObject);
begin
 if (TCheckBox(Sender).Checked) then
  begin
   gbRemoteParams.Enabled := False;
  end
 else
  begin
   gbRemoteParams.Enabled := True;
  end;
end;


procedure TfmMain.bnOpenTableClick(Sender: TObject);
begin
 if (EDBTable1.Active) then
  begin
    EDBTable1.Close;
    EDBTable1.Close;
    TButton(Sender).Caption := 'Open table';
  end
 else
  begin
    EDBDatabase1.Close;
    EDBSession1.Close;
    EDBDatabase1.SessionName := EDBSession1.SessionName;
    EDBTable1.SessionName := EDBSession1.SessionName;
    EDBSession1.RemoteHost := eHost.Text;
    EDBSession1.RemoteAddress := eHost.Text;
    EDBSession1.RemotePort := sePort.Value;
    EDBSession1.Connected := true;
    if (not IsDBExists) then
      CreateDB;
    EDBDatabase1.Open;
    EDBTable1.Open;
    DataSource1.DataSet := EDBTable1;
    TButton(Sender).Caption := 'Close table';
  end;
end;

procedure TfmMain.EDBTable1AfterClose(DataSet: TDataSet);
begin
  bnOpenTable.Caption := 'Open table';
  reLog.Lines.Add('Table closed');
end;

procedure TfmMain.EDBTable1AfterOpen(DataSet: TDataSet);
begin
  bnOpenTable.Caption := 'Close table';
  reLog.Lines.Add('Table opened, RecordCount = '+IntToStr(EDBTable1.RecordCount));
end;

procedure TfmMain.SetVCLEnabled(Value: Boolean);
begin
  if (Value) then
   begin
    EDBTable1.EnableControls;
   end
  else
   begin
    EDBTable1.DisableControls;
   end;
  bnInsert.Enabled := Value;
  bnEdit.Enabled := Value;
  bnDelete.Enabled := Value;
  bnRead.Enabled := Value;
  bnRecNo.Enabled := Value;
  seRecQty.Enabled := Value;
  eHost.Enabled := Value;
  sePort.Enabled := Value;
  reLog.Enabled := Value;
end;

procedure TfmMain.bnInsertClick(Sender: TObject);
var i,n,k,c,x:  Integer;
    t:          Cardinal;
    f:          Double;
begin
  if (not EDBTable1.Active) then
   bnOpenTableClick(bnOpenTable);
  SetVCLEnabled(False);
  t := GetTickCount;
  n := seRecQty.Value;
  try
    x := n div 100;
    c := EDBTable1.RecordCount;
    for i := 1 to n do
     begin
      if FClose then Exit;
      k := c+i;
      EDBTable1.Append;
      EDBTable1.FieldByName('Str').AsString := 'test_'+FormatFloat('000000000',k);
      EDBTable1.Post;
      if ((i mod x) = 0) then
       begin
        f := i / ((GetTickCount-t) + 0.000000001);
        lbProgress.Caption := IntToStr(i)+ ' '+FormatFloat('##0.000',f);
        Application.ProcessMessages;
       end;
     end;
  finally
    t := GetTickCount-t;
    f := t / 1000.0;
    reLog.Lines.Add('Insert '+IntToStr(n)+ ' records time, sec: '+#9+FormatFloat('#,##0.000',f));
    SetVCLEnabled(True);
    SaveLog;
  end;
end;

procedure TfmMain.bnCloseClick(Sender: TObject);
begin
  FClose := true;
  Close;
end;


procedure TfmMain.bnReadClick(Sender: TObject);
var i,n,x:      Integer;
    t:          Cardinal;
    f:          Double;
    s:          String;
begin
  if (not EDBTable1.Active) then
   bnOpenTableClick(bnOpenTable);
  SetVCLEnabled(False);
  t := GetTickCount;
  n := seRecQty.Value;
  try
    x := n div 100;
    EDBTable1.First;
    i := 0;
    while (not EDBTable1.Eof) and (i < n) do
     begin
      if FClose then Exit;
      s := EDBTable1.FieldByName('Str').AsString;
      Inc(i);
      EDBTable1.Next;
      if ((i mod x) = 0) then
       begin
        f := i / ((GetTickCount-t) + 0.000000001);
        lbProgress.Caption := IntToStr(i)+ ' '+FormatFloat('##0.000',f);
        Application.ProcessMessages;
       end;
     end;
  finally
    t := GetTickCount-t;
    f := t / 1000.0;
    reLog.Lines.Add('Read '+IntToStr(n)+ ' records time, sec: '+#9+FormatFloat('#,##0.000',f));
    SetVCLEnabled(True);
    SaveLog;
  end;
end;

procedure TfmMain.bnRecNoClick(Sender: TObject);
var i,n,x:      Integer;
    t:          Cardinal;
    f:          Double;
begin
  if (not EDBTable1.Active) then
   bnOpenTableClick(bnOpenTable);
  SetVCLEnabled(False);
  t := GetTickCount;
  n := seRecQty.Value;
  try
    x := n div 100;
    for i := 1 to n do
     begin
      if FClose then Exit;
      EDBTable1.RecNo := i;
      if ((i mod x) = 0) then
       begin
        f := i / ((GetTickCount-t) + 0.000000001);
        lbProgress.Caption := IntToStr(i)+ ' '+FormatFloat('##0.000',f);
        Application.ProcessMessages;
       end;
     end;
  finally
    t := GetTickCount-t;
    f := t / 1000.0;
    reLog.Lines.Add('RecNo '+IntToStr(n)+ ' records time, sec: '+#9+FormatFloat('#,##0.000',f));
    SetVCLEnabled(True);
    SaveLog;
  end;
end;

procedure TfmMain.bnEditClick(Sender: TObject);
var i,n,x:      Integer;
    t:          Cardinal;
    f:          Double;
    s:          String;
begin
  if (not EDBTable1.Active) then
   bnOpenTableClick(bnOpenTable);
  SetVCLEnabled(False);
  t := GetTickCount;
  n := seRecQty.Value;
  try
    x := n div 100;
    EDBTable1.First;
    i := 0;
    while (not EDBTable1.Eof) and (i < n) do
     begin
      if FClose then Exit;
      s := EDBTable1.FieldByName('Str').AsString;
      EDBTable1.Edit;
      EDBTable1.FieldByName('Str').AsString := s + ' - E';
      EDBTable1.Post;
      Inc(i);
      EDBTable1.Next;
      if ((i mod x) = 0) then
       begin
        f := i / ((GetTickCount-t) + 0.000000001);
        lbProgress.Caption := IntToStr(i)+ ' '+FormatFloat('##0.000',f);
        Application.ProcessMessages;
       end;
     end;
  finally
    t := GetTickCount-t;
    f := t / 1000.0;
    reLog.Lines.Add('Edit '+IntToStr(n)+ ' records time, sec: '+#9+FormatFloat('#,##0.000',f));
    SetVCLEnabled(True);
    SaveLog;
  end;
end;

procedure TfmMain.bnDeleteClick(Sender: TObject);
var i,n,x:      Integer;
    t:          Cardinal;
    f:          Double;
begin
  if (not EDBTable1.Active) then
   bnOpenTableClick(bnOpenTable);
  SetVCLEnabled(False);
  t := GetTickCount;
  n := seRecQty.Value;
  try
    x := n div 100;
    EDBTable1.First;
    i := 0;
    while (not EDBTable1.Eof) and (i < n) do
     begin
      if FClose then Exit;
      EDBTable1.Delete;
      Inc(i);
      if ((i mod x) = 0) then
       begin
        f := i / ((GetTickCount-t) + 0.000000001);
        lbProgress.Caption := IntToStr(i)+ ' '+FormatFloat('##0.000',f);
        Application.ProcessMessages;
       end;
     end;
  finally
    t := GetTickCount-t;
    f := t / 1000.0;
    reLog.Lines.Add('Delete '+IntToStr(n)+ ' records time, sec: '+#9+FormatFloat('#,##0.000',f));
    SetVCLEnabled(True);
    SaveLog;
  end;
end;

procedure TfmMain.SaveLog;
begin
  reLog.Lines.SaveToFile(IncludeTrailingBackslash(ExtractFilePath(ParamStr(0)))+'log.txt');
end;

function TfmMain.IsDBExists: Boolean;
begin
  try
    EDBQuery1.Close;
    EDBQuery1.DatabaseName := 'Configuration';
    EDBQuery1.SQL.Text := 'SELECT * FROM Databases WHERE Name = ' + AnsiQuotedStr(EDBDatabase1.DatabaseName,'''');
    EDBQuery1.RequestSensitive:=True;
    EDBQuery1.Open;
    Result := (EDBQuery1.RecordCount <> 0);
    EDBQuery1.Close;
  except
    Result := False;
  end;
end;

procedure TfmMain.CreateDB;
begin
  try
    EDBQuery1.Close;
    EDBQuery1.DatabaseName := 'Configuration';
    EDBQuery1.SQL.Text := 'CREATE DATABASE "TestDB" PATH ''.\data\''';
    EDBQuery1.ExecSQL;

    EDBQuery1.DatabaseName := EDBDatabase1.DatabaseName;
    EDBQuery1.SQL.Text := 'CREATE TABLE "test" ('
                          +#13#10+'"ID" INTEGER GENERATED ALWAYS AS IDENTITY (START WITH 0, INCREMENT BY 1),'
                          +#13#10+'"Str" VARCHAR(20)'
                          +#13#10+')';
    EDBQuery1.ExecSQL;
  except
   on e: Exception do
    ShowMessage('Cannot create database. Error: '+#13#10+e.Message);
  end;

{
      EDBTable1.IndexDefs.Clear;
      EDBTable1.FieldDefs.Clear;
      EDBTable1.FieldDefs.Add('ID',ftAutoInc);
      EDBTable1.FieldDefs.Add('Str',ftFixedChar,20);
      EDBTable1.IndexDefs.Add('PK','ID',[ixPrimary]);
      EDBTable1.IndexDefs.Add('Idx1','Str',[]);
      EDBTable1.CreateTable;
}
end;

end.
