unit uMain;

interface

{$I ver.inc}

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, DB, StdCtrls, ComCtrls, DBCtrls, Spin, ExtCtrls, Grids,
  DBGrids, nxdb, nxllComponent, nxtwWinsockTransport, nxllTransport,
  nxptBasePooledTransport, nxtnNamedPipeTransport, nxsdServerEngine,
  nxreRemoteServerEngine;

type
  TfmMain = class(TForm)
    DBGrid1: TDBGrid;
    Panel1: TPanel;
    lbProgress: TLabel;
    Label3: TLabel;
    bnInsert: TButton;
    bnClose: TButton;
    seRecQty: TSpinEdit;
    bnOpenTable: TButton;
    cbLocalDatabase: TCheckBox;
    bnRecNo: TButton;
    DBNavigator1: TDBNavigator;
    reLog: TRichEdit;
    DataSource1: TDataSource;
    bnEdit: TButton;
    bnDelete: TButton;
    bnRead: TButton;
    gbRemoteParams: TGroupBox;
    gbLocalParams: TGroupBox;
    Label1: TLabel;
    eHost: TEdit;
    Label2: TLabel;
    sePort: TSpinEdit;
    Label4: TLabel;
    eName: TEdit;
    Label5: TLabel;
    eFile: TEdit;
    bnBrowse: TButton;
    OpenDialog1: TOpenDialog;
    nxDatabase1: TnxDatabase;
    nxSession1: TnxSession;
    nxTable1: TnxTable;
    nxRemoteServerEngine1: TnxRemoteServerEngine;
    nxNamedPipeTransport1: TnxNamedPipeTransport;
    nxWinsockTransport1: TnxWinsockTransport;
    procedure FormCreate(Sender: TObject);
    procedure cbLocalDatabaseClick(Sender: TObject);
    procedure bnOpenTableClick(Sender: TObject);
    procedure nxTable1AfterClose(DataSet: TDataSet);
    procedure nxTable1AfterOpen(DataSet: TDataSet);
    procedure bnBrowseClick(Sender: TObject);
    procedure bnInsertClick(Sender: TObject);
    procedure bnCloseClick(Sender: TObject);
    procedure bnReadClick(Sender: TObject);
    procedure bnRecNoClick(Sender: TObject);
    procedure bnEditClick(Sender: TObject);
    procedure bnDeleteClick(Sender: TObject);
  private
    { Private declarations }
    FClose: Boolean;
  public
    { Public declarations }
    procedure SetVCLEnabled(Value: Boolean);
    procedure SaveLog;
  end;

var
  fmMain: TfmMain;

implementation

{$R *.dfm}

procedure TfmMain.FormCreate(Sender: TObject);
begin
 Caption := 'Client-Server Basic Speed Test. NexusDB version: '+nxTable1.Version+'. (c) AidAim Software, http://www.aidaim.com';
 FClose := False;
 nxDatabase1.Close;
end;

procedure TfmMain.cbLocalDatabaseClick(Sender: TObject);
begin
 if (TCheckBox(Sender).Checked) then
  begin
   gbLocalParams.Enabled := True;
   gbRemoteParams.Enabled := False;
  end
 else
  begin
   gbLocalParams.Enabled := False;
   gbRemoteParams.Enabled := True;
  end;
end;

procedure TfmMain.bnOpenTableClick(Sender: TObject);
begin
 if (nxTable1.Active) then
  begin
    nxTable1.Close;
    nxDatabase1.Close;
    TButton(Sender).Caption := 'Open table';
  end
 else
  begin
    nxDatabase1.Close;
    if (cbLocalDatabase.Checked) then
     begin
       // named pipes
       nxSession1.Active := False;
       nxDatabase1.AliasName := eName.Text;
       nxRemoteServerEngine1.Transport := nxNamedPipeTransport1;
       nxNamedPipeTransport1.ServerNameRuntime := eFile.Text;
       nxNamedPipeTransport1.Active := True;
     end
    else
     begin
       // TCP
       nxSession1.Active := False;
       nxDatabase1.AliasName := eName.Text;
       nxRemoteServerEngine1.Transport := nxWinsockTransport1;
       nxWinsockTransport1.ServerNameRuntime := eName.Text+'@'+ eHost.Text;
       nxWinsockTransport1.Port := Word(sePort.Value);
       nxWinsockTransport1.Active := True;
     end;
{
    nxDatabase1.LocalDatabase := cbLocalDatabase.Checked;
    nxDatabase1.DatabaseFileName := eFile.Text;
    nxDatabase1.ConnectionParams.RemoteHost := eHost.Text;
    nxDatabase1.ConnectionParams.RemotePort := sePort.Value;
    nxDatabase1.ConnectionParams.DatabaseName := eName.Text;
}
    nxDatabase1.Connected := true;
    if (not nxTable1.Exists) then
     begin
      nxTable1.FieldDefs.Clear;
      nxTable1.IndexDefs.Clear;
      nxTable1.FieldDefs.Add('ID',ftAutoInc);
      nxTable1.FieldDefs.Add('Str',ftFixedChar,20);
      nxTable1.IndexDefs.Add('PK','ID',[ixPrimary]);
      nxTable1.IndexDefs.Add('Idx1','Str',[]);
      nxTable1.CreateTable;
     end;
    nxTable1.Open;
    DataSource1.DataSet := nxTable1;
    TButton(Sender).Caption := 'Close table';
  end;
end;

procedure TfmMain.nxTable1AfterClose(DataSet: TDataSet);
begin
  bnOpenTable.Caption := 'Open table';
  reLog.Lines.Add('Table closed, Named Pipes = '+BoolToStr(cbLocalDatabase.Checked,True));
end;

procedure TfmMain.nxTable1AfterOpen(DataSet: TDataSet);
begin
  bnOpenTable.Caption := 'Close table';
  reLog.Lines.Add('Table opened, RecordCount = '+IntToStr(nxTable1.RecordCount)+', Named Pipes = '+BoolToStr(cbLocalDatabase.Checked,True));
end;

procedure TfmMain.bnBrowseClick(Sender: TObject);
begin
  if (OpenDialog1.Execute) then
    eFile.Text := OpenDialog1.FileName;
end;

procedure TfmMain.SetVCLEnabled(Value: Boolean);
begin
  if (Value) then
   begin
    nxTable1.EnableControls;
   end
  else
   begin
    nxTable1.DisableControls;
   end;
  bnInsert.Enabled := Value;
  bnEdit.Enabled := Value;
  bnDelete.Enabled := Value;
  bnRead.Enabled := Value;
  bnRecNo.Enabled := Value;
  bnBrowse.Enabled := Value;
  seRecQty.Enabled := Value;
  eFile.Enabled := Value;
  eHost.Enabled := Value;
  eName.Enabled := Value;
  sePort.Enabled := Value;
  reLog.Enabled := Value;
  cbLocalDatabase.Enabled := Value;
end;

procedure TfmMain.bnInsertClick(Sender: TObject);
var i,n,k,c,x:  Integer;
    t:          Cardinal;
    f:          Double;
begin
  if (not nxTable1.Active) then
   bnOpenTableClick(bnOpenTable);
  SetVCLEnabled(False);
  t := GetTickCount;
  n := seRecQty.Value;
  try
    x := n div 100;
    c := nxTable1.RecordCount;
    for i := 1 to n do
     begin
      if FClose then Exit;
      k := c+i;
      nxTable1.Append;
      nxTable1.FieldByName('Str').AsString := 'test_'+FormatFloat('000000000',k);
      nxTable1.Post;
      if ((i mod x) = 0) then
       begin
        f := i / ((GetTickCount-t) + 0.000000001);
        lbProgress.Caption := IntToStr(i)+ ' '+FormatFloat('##0.000',f);
        Application.ProcessMessages;
       end;
     end;
  finally
    t := GetTickCount-t;
    f := t / 1000.0;
    reLog.Lines.Add('Insert '+IntToStr(n)+ ' records time, sec: '+#9+FormatFloat('#,##0.000',f));
    SetVCLEnabled(True);
    SaveLog;
  end;
end;

procedure TfmMain.bnCloseClick(Sender: TObject);
begin
  FClose := true;
  Close;
end;

procedure TfmMain.SaveLog;
begin
  reLog.Lines.SaveToFile(IncludeTrailingBackslash(ExtractFilePath(ParamStr(0)))+'log.txt');
end;

procedure TfmMain.bnReadClick(Sender: TObject);
var i,n,x:      Integer;
    t:          Cardinal;
    f:          Double;
    s:          String;
begin
  if (not nxTable1.Active) then
   bnOpenTableClick(bnOpenTable);
  SetVCLEnabled(False);
  t := GetTickCount;
  n := seRecQty.Value;
  try
    x := n div 100;
    nxTable1.First;
    i := 0;
    while (not nxTable1.Eof) and (i < n) do
     begin
      if FClose then Exit;
      s := nxTable1.FieldByName('Str').AsString;
      Inc(i);
      nxTable1.Next;
      if ((i mod x) = 0) then
       begin
        f := i / ((GetTickCount-t) + 0.000000001);
        lbProgress.Caption := IntToStr(i)+ ' '+FormatFloat('##0.000',f);
        Application.ProcessMessages;
       end;
     end;
  finally
    t := GetTickCount-t;
    f := t / 1000.0;
    reLog.Lines.Add('Read '+IntToStr(n)+ ' records time, sec: '+#9+FormatFloat('#,##0.000',f));
    SetVCLEnabled(True);
    SaveLog;
  end;
end;

procedure TfmMain.bnRecNoClick(Sender: TObject);
var i,n,x:      Integer;
    t:          Cardinal;
    f:          Double;
begin
  if (not nxTable1.Active) then
   bnOpenTableClick(bnOpenTable);
  SetVCLEnabled(False);
  t := GetTickCount;
  n := seRecQty.Value;
  try
    x := n div 100;
    for i := 1 to n do
     begin
      if FClose then Exit;
      nxTable1.RecNo := i;
      if ((i mod x) = 0) then
       begin
        f := i / ((GetTickCount-t) + 0.000000001);
        lbProgress.Caption := IntToStr(i)+ ' '+FormatFloat('##0.000',f);
        Application.ProcessMessages;
       end;
     end;
  finally
    t := GetTickCount-t;
    f := t / 1000.0;
    reLog.Lines.Add('RecNo '+IntToStr(n)+ ' records time, sec: '+#9+FormatFloat('#,##0.000',f));
    SetVCLEnabled(True);
    SaveLog;
  end;
end;

procedure TfmMain.bnEditClick(Sender: TObject);
var i,n,x:      Integer;
    t:          Cardinal;
    f:          Double;
    s:          String;
begin
  if (not nxTable1.Active) then
   bnOpenTableClick(bnOpenTable);
  SetVCLEnabled(False);
  t := GetTickCount;
  n := seRecQty.Value;
  try
    x := n div 100;
    nxTable1.First;
    i := 0;
    while (not nxTable1.Eof) and (i < n) do
     begin
      if FClose then Exit;
      s := nxTable1.FieldByName('Str').AsString;
      nxTable1.Edit;
      nxTable1.FieldByName('Str').AsString := s + ' - Edited!';
      nxTable1.Post;
      Inc(i);
      nxTable1.Next;
      if ((i mod x) = 0) then
       begin
        f := i / ((GetTickCount-t) + 0.000000001);
        lbProgress.Caption := IntToStr(i)+ ' '+FormatFloat('##0.000',f);
        Application.ProcessMessages;
       end;
     end;
  finally
    t := GetTickCount-t;
    f := t / 1000.0;
    reLog.Lines.Add('Edit '+IntToStr(n)+ ' records time, sec: '+#9+FormatFloat('#,##0.000',f));
    SetVCLEnabled(True);
    SaveLog;
  end;
end;

procedure TfmMain.bnDeleteClick(Sender: TObject);
var i,n,x:      Integer;
    t:          Cardinal;
    f:          Double;
begin
  if (not nxTable1.Active) then
   bnOpenTableClick(bnOpenTable);
  SetVCLEnabled(False);
  t := GetTickCount;
  n := seRecQty.Value;
  try
    x := n div 100;
    nxTable1.First;
    i := 0;
    while (not nxTable1.Eof) and (i < n) do
     begin
      if FClose then Exit;
      nxTable1.Delete;
      Inc(i);
      if ((i mod x) = 0) then
       begin
        f := i / ((GetTickCount-t) + 0.000000001);
        lbProgress.Caption := IntToStr(i)+ ' '+FormatFloat('##0.000',f);
        Application.ProcessMessages;
       end;
     end;
  finally
    t := GetTickCount-t;
    f := t / 1000.0;
    reLog.Lines.Add('Delete '+IntToStr(n)+ ' records time, sec: '+#9+FormatFloat('#,##0.000',f));
    SetVCLEnabled(True);
    SaveLog;
  end;
end;

end.
